//
//  ViewController.m
//  BluetoothRCBot
//
//  Created by user on 4/7/15.
//  Copyright (c) 2015 Adrian Winata. All rights reserved.
//

#import "ViewController.h"

// Index for buffer
#define NORTH 0
#define EAST 2
#define SOUTH 1
#define WEST 3

@interface ViewController ()
@end

@implementation ViewController { }

@synthesize ble;

- (void)viewDidLoad
{
    [super viewDidLoad];
    
    self.analogStick.delegate = self;
    
    ble = [[BLE alloc]init];
    [ble controlSetup:1];
    ble.delegate = self;
    
    [self.connectButton addTarget:self action:@selector(scanForPeripherals:) forControlEvents:UIControlEventTouchUpInside];
    
    [self updateAnalogueLabel];
}

- (void)didReceiveMemoryWarning
{
    // Dispose of any resources that can be recreated.
    [super didReceiveMemoryWarning];
}


- (void)updateAnalogueLabel
{
	[self.analogTextLabel setText:[NSString stringWithFormat:@"Raw Analog Values: %.1f , %.1f", self.analogStick.xValue, self.analogStick.yValue]];
}

-(void)processAnalogControls
{
    if([ble isConnected]){
        // Buffer for BT data
        //UInt8 buf[9] = {0x03, 0x00, 0x05, 0x00, 0x06, 0x00, 0x0A, 0x00, 0x00};
        UInt8 buf[4] = {0x00, 0x00, 0x00, 0x00};
        
        // Scale -1.0 to 1.0 values between 0 and 255
        int scaledXval = abs(floorf((float)self.analogStick.xValue*255));
        int scaledYval = abs(floorf((float)self.analogStick.yValue*255));
        
        // Quadrant 1
        if(self.analogStick.xValue >= 0.0 && self.analogStick.yValue >= 0.0) {
            buf[NORTH] = scaledYval;
            buf[EAST] = scaledXval;
            buf[SOUTH] = 0x00;
            buf[WEST] = 0x00;
        }
        
        // Quadrant 2
        else if(self.analogStick.xValue <= 0.0 && self.analogStick.yValue >= 0.0) {
            buf[NORTH] = scaledYval;
            buf[EAST] = 0x00;
            buf[SOUTH] = 0x00;
            buf[WEST] = scaledXval;
        }
        
        // Quadrant 3
        else if(self.analogStick.xValue <= 0.0 && self.analogStick.yValue <= 0.0) {
            buf[NORTH] = 0x00;
            buf[EAST] = 0x00;
            buf[SOUTH] = scaledYval;
            buf[WEST] = scaledXval;
        }
        
        // Quadrant 4
        else if (self.analogStick.xValue >= 0.0 && self.analogStick.yValue <= 0.0) {
            buf[NORTH] = 0x00;
            buf[EAST] = scaledXval;
            buf[SOUTH] = scaledYval;
            buf[WEST] = 0x00;
        }
        
        // Writes buffer to BT
        NSData *d = [[NSData alloc]initWithBytes:buf length:4];
        [ble write:d];
    }
}

#pragma mark - JSAnalogueStickDelegate
- (void)analogueStickDidChangeValue:(JSAnalogueStick *)analogueStick
{
	[self updateAnalogueLabel];
    [self processAnalogControls];
}

#pragma mark - BLEDelegate
-(void)bleDidConnect
{
    [self.connectButton setTitle:@"Disconnect" forState:UIControlStateNormal];
    [self.connectButton removeTarget:self action:@selector(scanForPeripherals:) forControlEvents:UIControlEventTouchUpInside];
    [self.connectButton addTarget:self action:@selector(disconnectFromPeripheral) forControlEvents:UIControlEventTouchUpInside];
    [self.connectButton setEnabled:YES];
}

-(void)bleDidDisconnect
{
    [self.connectButton setTitle:@"Connect" forState:UIControlStateNormal];
    [self.connectButton removeTarget:self action:@selector(disconnectFromPeripheral) forControlEvents:UIControlEventTouchUpInside];
    [self.connectButton addTarget:self action:@selector(scanForPeripherals:) forControlEvents:UIControlEventTouchUpInside];
    self.rssiLabel.text = @"RSSI";
    [self.connectButton setEnabled:YES];
}

-(void)bleDidUpdateRSSI:(NSNumber *)rssi
{
    self.rssiLabel.text = [NSString stringWithFormat:@"RSSI:%@",rssi];
}

#pragma mark - BLE Actions
-(void)scanForPeripherals:(id)sender
{
    if(ble.peripherals) {
        ble.peripherals = nil;
    }
    [self.connectButton setEnabled:NO];
    [ble findBLEPeripherals:2];
    [NSTimer scheduledTimerWithTimeInterval:(float)2.0 target:self selector:@selector(connectionTimer:) userInfo:nil repeats:NO];
}

-(void)disconnectFromPeripheral
{
    [self.connectButton setEnabled:NO];
     if(ble.activePeripheral){
         if (ble.activePeripheral.isConnected){
             [[ble CM] cancelPeripheralConnection:[ble activePeripheral]];
         }
     }
}

-(void)connectionTimer:(NSTimer*)timer
{
    if(ble.peripherals.count > 0) {
        [ble connectPeripheral:[ble.peripherals objectAtIndex:0]];
    }
    else {
        self.connectButton.enabled = YES;
    }
}

@end
